if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 41478 $"):match("%d+"))

local CowTip = CowTip
local CowTip_Appearance = CowTip:NewModule("Appearance", "AceEvent-2.0")
local self = CowTip_Appearance
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-06-23 21:37:56 -0400 (Sat, 23 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip.desc = "Allow for a customized look of the tooltip"

local L = CowTip:L("CowTip_Appearance")

local SharedMedia = AceLibrary("SharedMedia-1.0")

local leftGameTooltipStrings = CowTip.leftGameTooltipStrings
local rightGameTooltipStrings = CowTip.rightGameTooltipStrings

local _G = _G
local unpack = _G.unpack
local GameTooltip = _G.GameTooltip
local UnitIsFriend = _G.UnitIsFriend
local GetFriendInfo = _G.GetFriendInfo
local GetNumFriends = _G.GetNumFriends
local UnitName = _G.UnitName
local UnitIsUnit = _G.UnitIsUnit
local UnitIsPlayer = _G.UnitIsPlayer
local GetGuildInfo = _G.GetGuildInfo
local UnitIsTappedByPlayer = _G.UnitIsTappedByPlayer
local UnitIsTapped = _G.UnitIsTapped
local UnitIsDeadOrGhost = _G.UnitIsDeadOrGhost
local GetWatchedFactionInfo = _G.GetWatchedFactionInfo

function CowTip_Appearance:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("Appearance")
	CowTip:RegisterDefaults("Appearance", "profile", {
		scale = 1,
		font = "Friz Quadrata TT",
		border = "Blizzard Tooltip",
		background = "Solid",
		bgColor = {
			guild = {0, 0.15, 0, 1},
			faction = {0.15, 0.15, 0, 1},
			hostilePC = {0.25, 0, 0, 1},
			hostileNPC = {0.15, 0, 0, 1},
			friendlyPC = {0, 0, 0.25, 1},
			friendlyNPC = {0, 0, 0.15, 1},
			other = {0, 0, 0, 1},
			dead = {0.15, 0.15, 0.15, 1},
			tapped = {0.25, 0.25, 0.25, 1},
		},
		borderColor = { 1, 1, 1, 1 },
		padding = 4,
		edgeSize = 4,
	})
end

function CowTip_Appearance:OnEnable()
	self:SetScale(nil)
	self:SetFont(nil)
	self:SetTexture(nil, nil, nil, nil)
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function CowTip_Appearance:OnDisable()
	self:SetScale(nil)
	self:SetFont(nil)
	self:SetTexture(nil, nil, nil, nil)
end

function CowTip_Appearance:PLAYER_ENTERING_WORLD()
	self:SetScale(nil)
end

function CowTip_Appearance:OnTooltipShow()
	self:SetBackgroundColor(nil)
	self:SetBorderColor(nil)
end

local currentSameFaction = false
function CowTip_Appearance:PreTooltipSetUnit()
	local myWatchedFaction = GetWatchedFactionInfo()
	currentSameFaction = false
	if myWatchedFaction then
		for i = 1, 10 do
			local left = leftGameTooltipStrings[i]
			if left:GetText() == myWatchedFaction then
				currentSameFaction = true
				break
			end
		end
	end
end

function CowTip_Appearance:SetScale(value)
	if value then
		self.db.profile.scale = value
	else
		value = self.db.profile.scale
	end
	if not CowTip:IsModuleActive(self) then
		value = 1
	end
	
	GameTooltip:SetScale(value)
end

function CowTip_Appearance:SetFont(value)
	if value then
		self.db.profile.font = value
	else
		value = self.db.profile.font
	end
	if not CowTip:IsModuleActive(self) then
		value = "Friz Quadrata TT"
	end
	
	local font = SharedMedia:Fetch('font', value)
	
	local text = leftGameTooltipStrings[1]
	if text:GetFont() == font then
		return
	end
	for i = 1, 50 do
		local left = leftGameTooltipStrings[i]
		local right = rightGameTooltipStrings[i]
		local _, size, style = left:GetFont()
		left:SetFont(font, size, style)
		local _, size, style = right:GetFont()
		right:SetFont(font, size, style)
	end
end

local tmp = {}
local tmp2 = {}
function CowTip_Appearance:SetTexture(border, background, padding, edgeSize)
	if border then
		self.db.profile.border = border
	else
		border = self.db.profile.border
	end
	if background then
		self.db.profile.background = background
	else
		background = self.db.profile.background
	end
	if padding then
		self.db.profile.padding = padding
	else
		padding = self.db.profile.padding
	end
	if edgeSize then
		self.db.profile.edgeSize = edgeSize
	else
		edgeSize = self.db.profile.edgeSize
	end
	if not CowTip:IsModuleActive(self) then
		border = "Blizzard Tooltip"
		background = "Blizzard Tooltip"
		padding = 4
		edgeSize = 16
	end
	
	tmp.bgFile = SharedMedia:Fetch('background', background)
	tmp.edgeFile = SharedMedia:Fetch('border', border)
	tmp.tile = false
	tmp.edgeSize = edgeSize
	tmp.insets = tmp2
	tmp2.left = padding
	tmp2.right = padding
	tmp2.top = padding
	tmp2.bottom = padding
	GameTooltip:SetBackdrop(tmp)
	
	self:SetBackgroundColor(nil)
	self:SetBorderColor(nil)
end

function CowTip_Appearance:SetBorder(value)
	self:SetTexture(value, nil, nil, nil)
end

function CowTip_Appearance:SetBackground(value)
	self:SetTexture(nil, value, nil, nil)
end

function CowTip_Appearance:SetPadding(value)
	self:SetTexture(nil, nil, value, nil)
end

function CowTip_Appearance:SetEdgeSize(value)
	self:SetTexture(nil, nil, nil, value)
end

function CowTip_Appearance:SetBackgroundColor(given_kind, r, g, b, a)
	local kind = given_kind
	if not kind then
		kind = 'other'
		local unit = CowTip.currentTooltipUnit
		if unit then
			if UnitIsDeadOrGhost(unit) then
				kind = 'dead'
			elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
				kind = 'tapped'
			elseif currentSameFaction then
				kind = 'faction'
			elseif UnitIsPlayer(unit) then
				if UnitIsFriend("player", unit) then
					local playerGuild = GetGuildInfo("player")
					if playerGuild and playerGuild == GetGuildInfo(unit) or UnitIsUnit("player", unit) then
						kind = 'guild'
					else
						local friend = false
						local name = UnitName(unit)
						for i = 1, GetNumFriends() do
							if GetFriendInfo(i) == name then
								friend = true
								break
							end
						end
						if friend then
							kind = 'guild'
						else
							kind = 'friendlyPC'
						end
					end
				else
					kind = 'hostilePC'
				end
			else
				if UnitIsFriend("player", unit) then
					kind = 'friendlyNPC'
				else
					kind = 'hostileNPC'
				end
			end
		end
	end
	local bgColor = self.db.profile.bgColor[kind]
	if r then
		bgColor[1] = r
		bgColor[2] = g
		bgColor[3] = b
		bgColor[4] = a
	else
		r, g, b, a = unpack(bgColor)
	end
	if given_kind then
		self:SetBackgroundColor(nil)
		return
	end
	if not CowTip:IsModuleActive(self) then
		r, g, b, a = 0, 0, 0, 1
	end
	GameTooltip:SetBackdropColor(r, g, b, a)
end

function CowTip_Appearance:SetBorderColor(r, g, b, a)
	local borderColor = self.db.profile.borderColor
	if r then
		borderColor[1] = r
		borderColor[2] = g
		borderColor[3] = b
		borderColor[4] = a
	else
		r, g, b, a = unpack(borderColor)
	end
	if not CowTip:IsModuleActive(self) then
		r, g, b, a = 1, 1, 1, 1
	end
	GameTooltip:SetBackdropBorderColor(r, g, b, a)
end

CowTip_Appearance:RegisterCowTipOption({
	name = L["Appearance"],
	desc = L["Change how the tooltip Appearance in general."],
	type = 'group',
	args = {
		bgColor = {
			name = L["Background color"],
			desc = L["Set what color the tooltip's background is."],
			type = 'group',
			pass = true,
			get = function(value)
				return unpack(CowTip_Appearance.db.profile.bgColor[value])
			end,
			set = "SetBackgroundColor",
			args = {
				guild = {
					name = L["Guild and friends"],
					desc = L["Background color for your guildmates and friends."],
					type = 'color',
					hasAlpha = true,
				},
				faction = {
					name = L["Currently watched faction"],
					desc = L["Background color for the currently watched faction."],
					type = 'color',
					hasAlpha = true,
				},
				hostilePC = {
					name = L["Hostile players"],
					desc = L["Background color for hostile players."],
					type = 'color',
					hasAlpha = true,
				},
				hostileNPC = {
					name = L["Hostile non-player characters"],
					desc = L["Background color for hostile non-player characters."],
					type = 'color',
					hasAlpha = true,
				},
				friendlyPC = {
					name = L["Friendly players"],
					desc = L["Background color for friendly players."],
					type = 'color',
					hasAlpha = true,
				},
				friendlyNPC = {
					name = L["Friendly non-player characters"],
					desc = L["Background color for friendly non-player characters."],
					type = 'color',
					hasAlpha = true,
				},
				dead = {
					name = L["Dead"],
					desc = L["Background color for dead units."],
					type = 'color',
					hasAlpha = true,
				},
				tapped = {
					name = L["Tapped"],
					desc = L["Background color for when a unit is tapped by another."],
					type = 'color',
					hasAlpha = true,
				},
				other = {
					name = L["Other"],
					desc = L["Background color for non-units."],
					type = 'color',
					hasAlpha = true,
				},
			}
		},
		borderColor = {
			name = L["Border color"],
			desc = L["Set what color the tooltip's border is."],
			type = 'color',
			hasAlpha = true,
			get = function()
				return unpack(CowTip_Appearance.db.profile.borderColor)
			end,
			set = "SetBorderColor"
		},
		border = {
			name = L["Border style"],
			desc = L["Change the border style."],
			type = 'text',
			validate = SharedMedia:List('border'),
			get = function()
				return CowTip_Appearance.db.profile.border
			end,
			set = "SetBorder",
		},
		background = {
			name = L["Background style"],
			desc = L["Change the background style.\n\nNote: You may need to change the Background color to white to see some of the backgrounds properly."],
			type = 'text',
			validate = SharedMedia:List('background'),
			get = function()
				return CowTip_Appearance.db.profile.background
			end,
			set = "SetBackground",
		},
		padding = {
			name = L["Border padding"],
			desc = L["The size of the border insets."],
			type = 'range',
			min = 0,
			max = 20,
			step = 1,
			get = function()
				return CowTip_Appearance.db.profile.padding
			end,
			set = "SetPadding",
		},
		edgeSize = {
			name = L["Border size"],
			desc = L["The size the border takes up."],
			type = 'range',
			min = 0,
			max = 20,
			step = 1,
			get = function()
				return CowTip_Appearance.db.profile.edgeSize
			end,
			set = "SetEdgeSize",
		},
		{
			name = L["Size"],
			desc = L["Set how large the tooltip is"],
			type = 'range',
			min = 0.25,
			max = 4,
			step = 0.01,
			bigStep = 0.05,
			get = function()
				return CowTip_Appearance.db.profile.scale
			end,
			set = "SetScale"
		},
		{
			name = L["Font"],
			desc = L["What font face to use."],
			type = 'text',
			validate = SharedMedia:List('font'),
			get = function()
				return CowTip_Appearance.db.profile.font
			end,
			set = "SetFont",
		},
	}
})
